/*****************************************************************************/
/* File: Ball.h                                                              */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 12/14/03 *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#if !defined(BALL_H)
#define BALL_H

#include "ModelNode.h"
#include <tokamak.h>

#ifdef _DEBUG
#include "Logger.h"
#endif 

/*****************************************************************************/
/*                            CLASS INTERFACE                                */
/*****************************************************************************/

namespace MidwayMath {

  class Ball : public ModelNode {

    public:
      
      // Constants
      static const float BALL_PHYSICS_MASS;
      static const float BALL_PHYSICS_VELOCITY;

      // Constructors / Destructors
      Ball(Renderer*, neSimulator*, SceneGraphNode*, const char *);
      
    #ifdef _DEBUG  
      Ball(Logger*, Renderer*, neSimulator*, SceneGraphNode*, const char *);
    #endif
      
      ~Ball();
      
      // Operations
      virtual void Update(); // Overrides parent's Update() 
      
      D3DXVECTOR3 GetPosition();
      void SetPosition(D3DXVECTOR3);
      void Throw(D3DXVECTOR3);
      inline int GetCollisionId() { return mpRigidBody->GetCollisionID(); }
      inline void SetCollisionId(int id) { mpRigidBody->SetCollisionID(id); }

    private:

      neSimulator *mpPhysics;
      neRigidBody *mpRigidBody;
  };

} // End Namespace
#endif
