/*****************************************************************************/
/* File: Crate.h                                                             */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 12/12/03 *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#if !defined(CRATE_H)
#define CRATE_H

#include "ModelNode.h"
#include <tokamak.h>

#ifdef _DEBUG
#include "Logger.h"
#endif 

/*****************************************************************************/
/*                            CLASS INTERFACE                                */
/*****************************************************************************/

namespace MidwayMath {

  class Crate : public ModelNode {

    public:
      
      //Constants
      static const float CRATE_PHYSICS_MASS;

      // Constructors / Destructors
      Crate(Renderer*, neSimulator*, SceneGraphNode*, const char *);
      Crate(Renderer*, neSimulator*, SceneGraphNode*, LPD3DXMESH);
      
    #ifdef _DEBUG  
      Crate(Logger*, Renderer*, neSimulator*, SceneGraphNode*, const char *);
      Crate(Logger*, Renderer*, neSimulator*, SceneGraphNode*, LPD3DXMESH);
    #endif
      
      ~Crate();
      
      //Operations
      virtual void Update(); // Overrides parent's Update() 
      virtual void Draw(); 
      
      D3DXVECTOR3 GetPosition();
      void SetPosition(D3DXVECTOR3);

      inline bool IsIdle()               { return (mpRigidBody->IsIdle() != 0 ? true : false); }
      inline int GetCollisionId()        { return mpRigidBody->GetCollisionID(); }
      inline void SetCollisionId(int id) { mpRigidBody->SetCollisionID(id); }
      inline int GetNumber()             { return miNumber; }     
      inline void SetNumber(int num)     { miNumber = num; }      
      inline void SetOnGround(bool g)    { mbOnGround = g; }      
      inline bool GetOnGround()          { return mbOnGround; }   

    private:

      void Initialize();
      neSimulator *mpPhysics;
      neRigidBody *mpRigidBody;
      CD3DFont *mpFont;
      D3DMATERIAL8 mGameFontMaterial;
      int miNumber;
      bool mbOnGround;
  };

} // End Namespace
#endif
