/*****************************************************************************/
/* File: InputManager.h                                                      */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* Processes user input for both menus and game play. Created by the main    */
/* application object (MidwayMathApp); this object is polled for input       */
/* returning an array of the predefined constants.                           */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 7/15/03  *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#if !defined(INPUTMANAGER_H)
#define INPUTMANAGER_H

#define DIRECTINPUT_VERSION 0x0800
#include <dinput.h>
#include <DXErr8.h>
#include "DIUtil.h"

#ifdef _DEBUG
#include "Logger.h"
#endif

/*****************************************************************************/
/*                            CLASS INTERFACE                                */
/*****************************************************************************/

namespace MidwayMath {

  class InputManager : public CInputDeviceManager {

    public:

      // Game-Play Constants (MS Visual C enum hack)
      enum { INPUT_PLAYER_LOOKUD        = 1,
             INPUT_PLAYER_LOOKLR,
             INPUT_PLAYER_FIRE,             
             INPUT_MENU_DEBUG,     
             INPUT_MENU_PAUSE,     
             INPUT_MENU_MOUSE_MOVE,       
             INPUT_MENU_UD,                             
             INPUT_MENU_UP,             
             INPUT_MENU_DOWN,           
             INPUT_MENU_SELECT,         
             INPUT_MENU_QUIT,           
             INPUT_MENU_WHEEL           
           };

      //Constructors / Destructors
      InputManager(HWND hWnd);
      
    #ifdef _DEBUG  
      InputManager(Logger *pLog, HWND hWnd);
    #endif

      ~InputManager();

      // Operations for polling the input state
      GetMouseState();
      int GetInputState(int aiUserInput[], int *piNumUserInput);
      int GetInputState();
      inline float GetAxisState() { return mfAxisState; }

      // Operations for setting input controls
      void ConfigureControls();
      inline void UseGameControls()  { SetActionFormat(mDIActionFormat, FALSE); }
      inline void UseMenuControls() { SetActionFormat(mDIMenuFormat, FALSE); }
      inline HWND GetHWND() { return mhWnd; }

    private:

      bool mabInput[15]; 
    
    #ifdef _DEBUG
      Logger *mpLog;
    #endif

      HWND mhWnd;
      float mfAxisState;
      DIACTIONFORMAT mDIActionFormat;
      DIACTIONFORMAT mDIMenuFormat;
    
      //Helper functions
      void Initialize();
  };

} // End Namespace
#endif
