/*****************************************************************************/
/* File: MenuItem.h                                                          */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 11/19/03  *   00   *  Austin  * Created.                                  */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#if !defined(MENUITEM_H)
#define MENUITEM_H

#include <vector>
#include <string>

#ifdef _DEBUG
#include "Logger.h"
#endif

/*****************************************************************************/
/*                            CLASS INTERFACE                                */
/*****************************************************************************/

namespace MidwayMath {

  class MenuItem {

    public:

      //Constructors / Destructors
      MenuItem() { mLabel = "NULL"; mlId = -1; mbSelected = false; }
      MenuItem(std::string text, long id) { mLabel = text; mlId = id; mbSelected = false; }
      
    #ifdef _DEBUG  
      MenuItem(Logger *plog, std::string text, long id) { mpLog = plog; mLabel = text; mlId = id; }
    #endif

      //~MenuItem();

      //Operations
      inline std::string GetLabel() { return mLabel; }
      inline long       GetId()    { return mlId; }
      inline void       Add(MenuItem *child) { mChildren.push_back(child); }
      inline int        GetNumberOfChildren() { return mChildren.size(); }
      inline MenuItem*  GetChild(int index) { return mChildren[index]; }

      inline void       Select()     { mbSelected = true; }
      inline void       Deselect()   { mbSelected = false; }
      inline bool       IsSelected() { return mbSelected; }

    private:

    #ifdef _DEBUG
      Logger *mpLog;
    #endif

      std::string mLabel;
      long        mlId; 
      std::vector<MenuItem*> mChildren;
      bool mbSelected;

  };

  /*****************************************************************************/
  /*                                 DESTRUCTOR                                */  
  /*****************************************************************************/

#if 0
  MenuItem::~MenuItem() {
    
    std::vector <MenuItem*>::iterator child;
    
      for(child = mChildren.begin(); child != mChildren.end(); child++) 
      delete child;
  }
#endif

} // End Namespace
#endif
