/*****************************************************************************/
/* File: MidwayMathUI.h                                                      */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 11/20/03  *   00   *  Austin  * Created.                                  */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#if !defined(MIDWAYMATHUI_H)
#define MIDWAYMATHUI_H

#include "UserInterface.h"

#ifdef _DEBUG
#include "Logger.h"
#endif

/*****************************************************************************/
/*                            CLASS INTERFACE                                */
/*****************************************************************************/

namespace MidwayMath {

  class MidwayMathUI : public UserInterface {

    public:
      
      static const long UI_ID_MAIN_PLAY;
      static const long UI_ID_MAIN_OPTIONS;
      static const long UI_ID_MAIN_INTRO;
      static const long UI_ID_MAIN_EXIT;
      static const long UI_ID_GAME_PAUSE;
      static const long UI_ID_GAME_QUIT;

      static const long UI_ID_OPTIONS_SOUND;
      static const long UI_ID_OPTIONS_VIDEO;
      static const long UI_ID_OPTIONS_BACK;

      static const char* const UI_TEXTURE_MAIN_MENU;

      // Constructors / Destructors
      MidwayMathUI(Renderer *r);
    
    #ifdef _DEBUG
      MidwayMathUI(Logger *l, Renderer *r);
    #endif
      
      ~MidwayMathUI();
      
      // Operations
      virtual void DrawMenu(long id);
      virtual void DrawPauseMenu();
      virtual void DrawQuitMenu();
      virtual void DrawHUD();   

      virtual void MenuDown(long id);
      virtual void MenuUp(long id);
      virtual MenuItem* GetSelected(long id);

      void SetProblem(const char *p) { mProblem = p; }
      void SetStat1(const char *s) { mStats1 = s; }
      void SetStat2(const char *s) { mStats2 = s; }
      void SetStat3(const char *s) { mStats3 = s; }
      
     void DarkenScene(float);
    
    private:

      struct SCREENVERTEX {
        D3DXVECTOR4 p;
        DWORD       color;
      };

      // Helper Functions
      void CreateFonts();
      void CreateImages();
    
      float mfTitlePosition;
      LPD3DXMESH   mpTitleText;
      LPDIRECT3DSURFACE8 mpBackgroundImage;
      LPDIRECT3DVERTEXBUFFER8 mpViewport;
      D3DMATERIAL8 mTitleMaterial;
      D3DMATERIAL8 mNormalMaterial;
      D3DMATERIAL8 mSelectedMaterial;
      D3DMATERIAL8 mGameFontMaterial;
      CD3DFont *mpFont;
      std::string mProblem;
      std::string mStats1;
      std::string mStats2;
      std::string mStats3;
  };

} // End Namespace
#endif
  