/*****************************************************************************/
/* File: ModelFileLoader.h                                                   */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* Loads an .x model file into a ModelNode object. It is important to note   */
/* that this implementation of a model loader will only operate correctly    */
/* if the model is exported from 3D Studio MAX with a 'top-level' hierarchy  */
/* _only_. This means that meshes more than one level deep will not load     */
/* correctly and may lead to unpredictable results. Care should be take when */
/* exporting the model in .x format.                                         */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 6/15/03  *   01   *  Austin  * Misc. Cleanup                              */
/*---------------------------------------------------------------------------*/
/* 4/27/03  *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#if !defined(MODELFILELOADER_H)
#define MODELFILELOADER_H

#include "FileLoader.h"
#include "ModelNode.h"
#include "Renderer.h"

#ifdef _DEBUG
#include "Logger.h"
#endif 

/*****************************************************************************/
/*                            CLASS INTERFACE                                */
/*****************************************************************************/

namespace MidwayMath {

  class ModelFileLoader : public FileLoader {

    public:

      // Constructors / Destructors
      ModelFileLoader() { }
      
    #ifdef _DEBUG  
       ModelFileLoader(Logger *, Renderer *, const char *, ModelNode *);
    #endif
       ModelFileLoader(Renderer *, const char *, ModelNode *);
      ~ModelFileLoader();

      // Operations
      inline void Load(SceneGraphNode*);
      inline void Load(char *file);
      void Load();
    
    private:
     
      Renderer  *mpRenderer;
      
      // Current node in the hierarchy
      ModelNode *mpCurrentNode;
      
      // Helper functions
      void LoadModelData(LPDIRECTXFILEDATA);
      void LoadModelMesh(LPDIRECTXFILEDATA);
  };

} // End Namespace
#endif
