/*****************************************************************************/
/* File: ModelNode.h                                                         */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* A model node in the Scene Graph. These are populated by the               */
/* ModelFileLoader class.                                                    */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 6/15/03  *   02   *  Austin  * Misc. cleanup                              */
/*---------------------------------------------------------------------------*/
/* 5/10/03  *   01   *  Austin  * Added operations from 2nd iteration design.*/
/*---------------------------------------------------------------------------*/
/* 4/27/03  *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#if !defined(MODELNODE_H)
#define MODELNODE_H

#include <D3dx8mesh.h>
#include "d3dutil.h"
#include "Renderer.h"
#include "SceneGraphNode.h"
//#include "Controller.h"

#ifdef _DEBUG
#include "Logger.h"
#endif 

/*****************************************************************************/
/*                            CLASS INTERFACE                                */
/*****************************************************************************/

namespace MidwayMath {

  class ModelNode : public SceneGraphNode {

    public:
      
      // Constructors / Destructors
      ModelNode();
      ModelNode(Renderer *r, char *n) 
        { mpMesh = NULL; mpRenderer = r; Initialize(n); }
      ModelNode(Renderer *r, char *n, SceneGraphNode *p)
        { mpMesh = NULL; mpRenderer = r; Initialize(n,p); }
    #ifdef _DEBUG
      ModelNode(Logger *l, Renderer *r, char *n); 
      ModelNode(Logger *l, Renderer *r, char *n, SceneGraphNode *p); 
    #endif
      ~ModelNode();
      
      // Operations
      inline LPD3DXMESH GetMesh()      { return mpMesh; }
      inline LPD3DXBUFFER GetMatBuf() { return mpMaterialBuffer; }
      inline LPD3DXBUFFER GetAdjBuf() { return mpAdjacencyBuffer; }
      inline DWORD GetNumMaterials()  { return mlNumMaterials; }
      inline LPDIRECT3DTEXTURE8* GetTextures()  { return mpTextures; }

      inline void SetMesh(LPD3DXMESH m)      { mpMesh = m; }
      inline void SetMatBuf(LPD3DXBUFFER b) { mpMaterialBuffer = b; }
      inline void SetAdjBuf(LPD3DXBUFFER b) { mpAdjacencyBuffer = b; }
      inline void SetNumMaterials(DWORD n)  { mlNumMaterials = n; }
      inline void SetTextures(LPDIRECT3DTEXTURE8 *t) { mpTextures = t; }    
      
      virtual void Draw();
      virtual void Update();    

      // Matrix Data
      FLOAT       mfLocalScale; 
      D3DXMATRIX  mLocalRotation;
      D3DXVECTOR3 mLocalTranslation;

      FLOAT       mfWorldScale; 
      D3DXMATRIX  mWorldRotation;
      D3DXVECTOR3 mWorldTranslation;

    protected:
      // Renderer
      Renderer *mpRenderer; 
    
    private:

      // Model Data
      LPD3DXMESH mpMesh;
      LPD3DXBUFFER  mpMaterialBuffer; 
      LPD3DXBUFFER  mpAdjacencyBuffer;
      LPDIRECT3DTEXTURE8 *mpTextures;
      DWORD mlNumMaterials;
  
      D3DXMATRIX mpWorldMatrix; 
      /* Is this needed?
      D3DXMATRIX mpLocalMatrix; */

      // Controllers
      //std::vector<Controller> mControllers;
  };

} // End Namespace
#endif
