/*****************************************************************************/
/* File: UserInterface.h                                                     */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 11/19/03  *   00   *  Austin  * Created.                                  */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#if !defined(USERINTERFACE_H)
#define USERINTERFACE_H

#include <vector>
#include <string>
#include "MenuItem.h"
#include "Renderer.h"

#ifdef _DEBUG
#include "Logger.h"
#endif

/*****************************************************************************/
/*                            CLASS INTERFACE                                */
/*****************************************************************************/

namespace MidwayMath {

  class UserInterface {

    public:

      //Pseudo-Constructors 
      inline void Initialize(Renderer *r) { mpRenderer = r; }
      
    #ifdef _DEBUG  
      inline void Initialize(Logger *l, Renderer *r) { mpLog = l; mpRenderer = r; }
    #endif

      //Operations
      inline virtual MenuItem* AddMenuItem(std::string text, long id);
      inline virtual MenuItem* AddMenuItem(MenuItem *parent, std::string text, long id);
      inline virtual void      AddSubMenu(MenuItem *mainMenu, MenuItem *subMenu);

      virtual void DrawMenu(long id) = 0;
      virtual void MenuDown(long id) = 0;
      virtual void MenuUp(long id)   = 0;
      virtual MenuItem* GetSelected(long id) = 0;

      virtual void DrawHUD()         = 0;

    protected:

      // Helper functions
      inline MenuItem *FindSubMenu(long id);

      // Renderer
      Renderer *mpRenderer; 
      
      std::vector<MenuItem*> mSubMenus;

    #ifdef _DEBUG
      Logger *mpLog;
    #endif

  };

  /****************************************************************************/
  /*  FUNCTION:                                                               */
  /*                                                                          */
  /*      AddMenuItem                                                         */
  /*                                                                          */
  /*  PARAMETERS:                                                             */
  /*                                                                          */
  /*  DESCRIPTION:                                                            */
  /*                                                                          */
  /*  RETURNS:                                                                */
  /*                                                                          */
  /****************************************************************************/

  inline MenuItem* UserInterface::AddMenuItem(std::string text, long id) {

  MenuItem *item = new MenuItem(text, id);
  mSubMenus.push_back(item);
  return item;
  }

  /****************************************************************************/
  /*  FUNCTION:                                                               */
  /*                                                                          */
  /*      AddMenuItem                                                         */
  /*                                                                          */
  /*  PARAMETERS:                                                             */
  /*                                                                          */
  /*  DESCRIPTION:                                                            */
  /*                                                                          */
  /*  RETURNS:                                                                */
  /*                                                                          */
  /****************************************************************************/

  inline MenuItem* UserInterface::AddMenuItem(MenuItem *parent, std::string text, long id) {

  MenuItem *item = new MenuItem(text, id);
  parent->Add(item);
  return item;
  }

  /****************************************************************************/
  /*  FUNCTION:                                                               */
  /*                                                                          */
  /*      AddSubMenu                                                          */
  /*                                                                          */
  /*  PARAMETERS:                                                             */
  /*                                                                          */
  /*  DESCRIPTION:                                                            */
  /*                                                                          */
  /*  RETURNS:                                                                */
  /*                                                                          */
  /****************************************************************************/

  inline void UserInterface::AddSubMenu(MenuItem *mainMenu, MenuItem *subMenu) {

  mainMenu->Add(subMenu);
  }

  /****************************************************************************/
  /*  FUNCTION:                                                               */
  /*                                                                          */
  /*      FindSubMenu                                                         */
  /*                                                                          */
  /*  PARAMETERS:                                                             */
  /*                                                                          */
  /*  DESCRIPTION:                                                            */
  /*                                                                          */
  /*  RETURNS:                                                                */
  /*                                                                          */
  /****************************************************************************/

  inline MenuItem* UserInterface::FindSubMenu(long id) {

    std::vector <MenuItem*>::iterator menu;
    
      for(menu = mSubMenus.begin(); menu != mSubMenus.end(); menu++) {

        if((*menu)->GetId() == id)
          return *menu;
      }
    
    return NULL;
  }

} // End Namespace
#endif
