/*****************************************************************************/
/* File: Ground.cpp                                                          */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 12/14/03 *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#include "Ground.h"
#include "ModelFileLoader.h"

using namespace MidwayMath;

/*****************************************************************************/
/*                               CONSTRUCTORS                                */  
/*****************************************************************************/

Ground::Ground(Logger *l, Renderer *r, neSimulator *sim, const char *file) 
  :ModelNode(l, r, "Ground"),
   mpPhysics(sim) {

LPD3DXMESH mesh;

// Bounding box stuff
BYTE* pVertices;
DWORD fvf;
DWORD numVert;
D3DXVECTOR3 min, max;
neGeometry *geom;
neV3 position;
float x, y, z;

  // Load the mesh
  ModelFileLoader *loader = new ModelFileLoader(l, r, file, this);
  loader->Load();
  delete loader;

  mesh = GetMesh();
  fvf = mesh->GetFVF();
  numVert = mesh->GetNumVertices();

  // Compute the bounding box
  mesh->LockVertexBuffer(D3DLOCK_READONLY, &pVertices);

    D3DXComputeBoundingBox(pVertices,
                           numVert,
                           fvf,
                           &min,
                           &max);

  mesh->UnlockVertexBuffer();

  x = max.x - min.x;
  y = max.y - min.y;
  z = max.z - min.z;

  // Create the animated body
  mpAnimatedBody = mpPhysics->CreateAnimatedBody();

  // Set the bounding box dimensions
  geom = mpAnimatedBody->AddGeometry();
  geom->SetBoxSize(x, y, z);

  // Set the position of the box
  position.Set(0.0f, 0.0f, 0.0f);
  mpAnimatedBody->SetPos(position);
  mpAnimatedBody->UpdateBoundingInfo();
}

/*****************************************************************************/
/*                               DESTRUCTOR                                  */  
/*****************************************************************************/

Ground::~Ground() {

  // Free the physics stuff
  mpPhysics->FreeAnimatedBody(mpAnimatedBody);
}
