/*****************************************************************************/
/* File: ModelNode.cpp                                                       */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* A model node in the Scene Graph. These are populated by the               */
/* ModelFileLoader class.                                                    */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 5/10/03  *   01   *  Austin  * Added operations from 2nd iteration design.*/
/*---------------------------------------------------------------------------*/
/* 4/29/03  *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#include "ModelNode.h"

using namespace MidwayMath;

/*****************************************************************************/
/*                               CONSTRUCTORS                                */  
/*****************************************************************************/

ModelNode::ModelNode()
    :mpMesh(NULL),
     mpRenderer(NULL) {

  Initialize();

  mfLocalScale = 1.0f; 
  D3DXMatrixIdentity(&mLocalRotation);
  mLocalTranslation.x = 0.0f;
  mLocalTranslation.y = 0.0f;
  mLocalTranslation.z = 0.0f;

  mfWorldScale = 1.0f; 
  D3DXMatrixIdentity(&mWorldRotation);
  mWorldTranslation.x = 0.0f;
  mWorldTranslation.y = 0.0f;
  mWorldTranslation.z = 0.0f;
}

ModelNode::ModelNode(Logger *l, Renderer *r, char *n) 
    :mpMesh(NULL),
     mpRenderer(r) {

  Initialize(n,l);

  mfLocalScale = 1.0f; 
  D3DXMatrixIdentity(&mLocalRotation);
  mLocalTranslation.x = 0.0f;
  mLocalTranslation.y = 0.0f;
  mLocalTranslation.z = 0.0f;

  mfWorldScale = 1.0f; 
  D3DXMatrixIdentity(&mWorldRotation);
  mWorldTranslation.x = 0.0f;
  mWorldTranslation.y = 0.0f;
  mWorldTranslation.z = 0.0f;
}

ModelNode::ModelNode(Logger *l, Renderer *r, char *n, SceneGraphNode *p) 
    :mpMesh(NULL),
     mpRenderer(r) {

  Initialize(n,l,p);

  mfLocalScale = 1.0f; 
  D3DXMatrixIdentity(&mLocalRotation);
  mLocalTranslation.x = 0.0f;
  mLocalTranslation.y = 0.0f;
  mLocalTranslation.z = 0.0f;

  mfWorldScale = 1.0f; 
  D3DXMatrixIdentity(&mWorldRotation);
  mWorldTranslation.x = 0.0f;
  mWorldTranslation.y = 0.0f;
  mWorldTranslation.z = 0.0f;
}

/*****************************************************************************/
/*                               DESTRUCTOR                                  */  
/*****************************************************************************/

ModelNode::~ModelNode() {

}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      Draw                                                                 */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Draws the model and associated texture. This is called from the      */
/*      Renderer object or from a parent node in the Scene Graph.            */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void ModelNode::Draw() {

  if(!GetMesh()) {

  for(int i = 0; i < GetNumberOfChildren(); i++)
    GetChild(i)->Draw();
  
  return;
  }
D3DXMATERIAL *materials;

  if(mlNumMaterials > 0)
    materials = (D3DXMATERIAL*) mpMaterialBuffer->GetBufferPointer();

//  mpRenderer->GetDevice()->SetRenderState(D3DRS_LIGHTING, TRUE);
//  mpRenderer->GetDevice()->LightEnable(0,TRUE);
//  mpRenderer->GetDevice()->SetRenderState(D3DRS_AMBIENT, D3DCOLOR_XRGB(255,255,255));

  mpRenderer->GetDevice()->SetTransform(D3DTS_WORLD, &mpWorldMatrix);

#if 0  
  D3DLIGHT8 light;
  D3DUtil_InitLight(light, D3DLIGHT_DIRECTIONAL);

  light.Diffuse.r     = 255;
  light.Diffuse.g     = 255;
  light.Diffuse.b     = 255;
  
  light.Direction = D3DXVECTOR3(0,-1,0);
  mpRenderer->GetDevice()->SetLight(0,&light);
  
  light.Direction = D3DXVECTOR3(0,1,0);
  mpRenderer->GetDevice()->SetLight(1,&light);
  
  light.Direction = D3DXVECTOR3(1,0,0);
  mpRenderer->GetDevice()->SetLight(2,&light);
  
  light.Direction = D3DXVECTOR3(-1,0,0);
  mpRenderer->GetDevice()->SetLight(3,&light);
  
  light.Direction = D3DXVECTOR3(0,0,1);
  mpRenderer->GetDevice()->SetLight(4,&light);
  
  light.Direction = D3DXVECTOR3(0,0,-1);
  mpRenderer->GetDevice()->SetLight(5,&light);
  
  mpRenderer->GetDevice()->LightEnable(0, true);
  mpRenderer->GetDevice()->LightEnable(1, true);
  mpRenderer->GetDevice()->LightEnable(2, true);
  mpRenderer->GetDevice()->LightEnable(3, true);
  mpRenderer->GetDevice()->LightEnable(4, true);
  mpRenderer->GetDevice()->LightEnable(5, true);
#endif
  
#if 0
  struct LitPointVertex
  {
    float x, y, z;
    DWORD color;
  };
  
  LitPointVertex x[2];
  LitPointVertex y[2];
  LitPointVertex z[2];
  x[0].color = D3DCOLOR_XRGB(0,0,0);
  x[1].color = D3DCOLOR_XRGB(0,0,0);
  y[0].color = D3DCOLOR_XRGB(0,0,0);
  y[1].color = D3DCOLOR_XRGB(0,0,0);
  z[0].color = D3DCOLOR_XRGB(0,0,0);
  z[1].color = D3DCOLOR_XRGB(0,0,0);

  x[0].x = x[0].y = x[0].z = 0; 
  y[0].x = y[0].y = y[0].z = 0; 
  z[0].x = z[0].y = z[0].z = 0;
  
  // x axis
  x[1].x = 5;   x[1].y = 0; x[1].z = 0;
  // y axis
  y[1].x = 0;   y[1].y = 5; y[1].z = 0;
  // z axis
  z[1].x = 0;   z[1].y = 0; z[1].z = 5;

  mpRenderer->GetDevice()->DrawPrimitiveUP(D3DPT_LINELIST, 1, x, sizeof(LitPointVertex));
  mpRenderer->GetDevice()->DrawPrimitiveUP(D3DPT_LINELIST, 1, y, sizeof(LitPointVertex));
  mpRenderer->GetDevice()->DrawPrimitiveUP(D3DPT_LINELIST, 1, z, sizeof(LitPointVertex));
#endif
  
  for(int i = 0; i < mlNumMaterials; i++) {

    mpRenderer->GetDevice()->SetMaterial(&materials[i].MatD3D);
    if(mpTextures && mpTextures[i])
      mpRenderer->GetDevice()->SetTexture(0, mpTextures[i]);
    mpMesh->DrawSubset(i);
  }
  
  if(mlNumMaterials == 0)
    mpMesh->DrawSubset(0);
  
  for(i = 0; i < GetNumberOfChildren(); i++)
    GetChild(i)->Draw();
}


/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      Update                                                               */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void ModelNode::Update() {

ModelNode *parent = dynamic_cast<ModelNode*>(GetParent());

  //update the world transformation (i.e. concatenate the parent world matrix with node local matrix)
  if(parent) {

    //concatenate the scaling factor
    mfWorldScale = parent->mfWorldScale * mfLocalScale;

    //concatenate the rotation matrices
    //D3DXMatrixMultiply(&mWorldRotation, &parent->mWorldRotation,&mLocalRotation); 
    //D3DXMatrixMultiply(&mWorldRotation, &mLocalRotation, &parent->mWorldRotation);    
    mWorldRotation = mLocalRotation * parent->mWorldRotation;   
  
    //rotate the translation vector 
    D3DXVECTOR3 t;     
    D3DXVec3TransformCoord(&t,&mLocalTranslation,&parent->mWorldRotation);

    //add the translations
    mWorldTranslation = parent->mWorldTranslation + (parent->mfWorldScale * t);
  }
  
  else {
  
    mfWorldScale      = mfLocalScale;
    mWorldRotation    = mLocalRotation;
    mWorldTranslation = mLocalTranslation;
  }

  mpWorldMatrix._11 = mfWorldScale * mWorldRotation._11; mpWorldMatrix._12 = mfWorldScale * mWorldRotation._12; mpWorldMatrix._13 = mfWorldScale * mWorldRotation._13; mpWorldMatrix._14 = 0.0f;
  mpWorldMatrix._21 = mfWorldScale * mWorldRotation._21; mpWorldMatrix._22 = mfWorldScale * mWorldRotation._22; mpWorldMatrix._23 = mfWorldScale * mWorldRotation._23; mpWorldMatrix._24 = 0.0f;
  mpWorldMatrix._31 = mfWorldScale * mWorldRotation._31; mpWorldMatrix._32 = mfWorldScale * mWorldRotation._32; mpWorldMatrix._33 = mfWorldScale * mWorldRotation._33; mpWorldMatrix._34 = 0.0f;
  mpWorldMatrix._41 = mWorldTranslation.x;               mpWorldMatrix._42 = mWorldTranslation.y;               mpWorldMatrix._43 = mWorldTranslation.z;               mpWorldMatrix._44 = 1.0f;

  for(int i = 0; i < GetNumberOfChildren(); i++)
    GetChild(i)->Update();
}


