/*****************************************************************************/
/* File: WinMain.cpp                                                         */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* Contains WinMain and start up code to boot strap the rest of the program. */
/* Creates a global application class which is passed any Windows' Messages  */
/* recieved from the message callback function.                              */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 6/15/03 *   01   *  Austin  * Misc. cleanup                               */
/*---------------------------------------------------------------------------*/
/* 4/31/03 *   00   *  Austin  * Created.                                    */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include "MidwayMathApp.h"

using namespace MidwayMath;

/*****************************************************************************/
/*                                 GLOBALS                                   */  
/*****************************************************************************/

MidwayMathApp *gpMainApp;

/*****************************************************************************/
/*                               FUNCTIONALITY                               */  
/*****************************************************************************/

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      WinMain                                                              */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*      hInstance                                                            */
/*          Handle to the current instance of the application.               */
/*                                                                           */
/*      hPrevInst                                                            */
/*          Handle to the previous instance of the application. For a        */
/*          Win32-based application, this parameter is always NULL.          */
/*                                                                           */
/*      pCmdLine                                                             */
/*          Pointer to a null-terminated string that specifies the command   */
/*          line for the application, excluding the program name.            */
/*                                                                           */
/*      nShowCmd                                                             */
/*          Specifies how the window is to be shown.                         */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Entry point to the program.                                          */ 
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*      The exit value contained in that messages wParam parameter          */
/*      indicates success, and that the function terminates when it receives */
/*      a WM_QUIT message. Zero indicates that the function terminates       */
/*      before entering the message loop.                                    */
/*****************************************************************************/

int APIENTRY WinMain(HINSTANCE hInstance, 
                     HINSTANCE hPrevInst, 
                     LPSTR pCmdLine, 
                     int nCmdShow) {

  
  try {
  
    gpMainApp = new MidwayMathApp(hInstance);
  }
  
  catch (char *exception){

    // Simply refuse to run with any initialization exceptions
    MessageBox(NULL, exception, "Midway Math Error", MB_ICONERROR);
    return 1;
  }
  
  // Run() will return when the application is closed
  gpMainApp->Run();
  
  delete gpMainApp;

  return 0;
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      WinMessageHandler                                                    */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*      hWnd                                                                 */
/*          Handle to the current window.                                    */
/*                                                                           */
/*      msg                                                                  */
/*          Message identifier.                                              */
/*                                                                           */
/*      wParam                                                               */
/*          First message parameter.                                         */
/*                                                                           */
/*      lParam                                                               */
/*          Second message parameter.                                        */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Message handler function. Process Windows messages by calling        */
/*      appropriate application routines.                                    */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*      LRESULT                                                              */
/*          Depends on the message sent.                                     */
/*                                                                           */
/*****************************************************************************/

LRESULT CALLBACK WinMessageHandler(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {

  return gpMainApp->MessageHandler(hWnd, msg, wParam, lParam);
}
